import { type BoundKind, type LimitLiteral, type RawSchema } from "@arktype/schema";
import { type keySet } from "@arktype/util";
import type { astToString } from "../../../semantic/utils.js";
import type { DynamicState, DynamicStateWithRoot } from "../../reduce/dynamic.js";
import { type Comparator, type InvertedComparators, type MaxComparator, writeUnpairableComparatorMessage } from "../../reduce/shared.js";
import type { StaticState, state } from "../../reduce/static.js";
import type { parseOperand } from "../operand/operand.js";
import type { Scanner } from "../scanner.js";
export declare const parseBound: (s: DynamicStateWithRoot, start: ComparatorStartChar) => void;
export type parseBound<s extends StaticState, start extends ComparatorStartChar, unscanned extends string, $, args> = shiftComparator<start, unscanned> extends infer shiftResultOrError ? shiftResultOrError extends (Scanner.shiftResult<infer comparator extends Comparator, infer nextUnscanned>) ? s["root"] extends `${infer limit extends LimitLiteral}` ? state.reduceLeftBound<s, limit, comparator, nextUnscanned> : parseRightBound<state.scanTo<s, nextUnscanned>, comparator, $, args> : shiftResultOrError : never;
declare const oneCharComparators: {
    readonly "<": true;
    readonly ">": true;
};
type OneCharComparator = keyof typeof oneCharComparators;
export type ComparatorStartChar = Comparator extends `${infer char}${string}` ? char : never;
export declare const comparatorStartChars: keySet<ComparatorStartChar>;
declare const shiftComparator: (s: DynamicState, start: ComparatorStartChar) => Comparator;
type shiftComparator<start extends ComparatorStartChar, unscanned extends string> = unscanned extends `=${infer nextUnscanned}` ? [`${start}=`, nextUnscanned] : start extends OneCharComparator ? [start, unscanned] : state.error<singleEqualsMessage>;
export declare const writeIncompatibleRangeMessage: (l: BoundKind, r: BoundKind) => "Bound kinds exactLength and exactLength are incompatible" | "Bound kinds exactLength and max are incompatible" | "Bound kinds exactLength and min are incompatible" | "Bound kinds exactLength and maxLength are incompatible" | "Bound kinds exactLength and minLength are incompatible" | "Bound kinds exactLength and before are incompatible" | "Bound kinds exactLength and after are incompatible" | "Bound kinds max and exactLength are incompatible" | "Bound kinds max and max are incompatible" | "Bound kinds max and min are incompatible" | "Bound kinds max and maxLength are incompatible" | "Bound kinds max and minLength are incompatible" | "Bound kinds max and before are incompatible" | "Bound kinds max and after are incompatible" | "Bound kinds min and exactLength are incompatible" | "Bound kinds min and max are incompatible" | "Bound kinds min and min are incompatible" | "Bound kinds min and maxLength are incompatible" | "Bound kinds min and minLength are incompatible" | "Bound kinds min and before are incompatible" | "Bound kinds min and after are incompatible" | "Bound kinds maxLength and exactLength are incompatible" | "Bound kinds maxLength and max are incompatible" | "Bound kinds maxLength and min are incompatible" | "Bound kinds maxLength and maxLength are incompatible" | "Bound kinds maxLength and minLength are incompatible" | "Bound kinds maxLength and before are incompatible" | "Bound kinds maxLength and after are incompatible" | "Bound kinds minLength and exactLength are incompatible" | "Bound kinds minLength and max are incompatible" | "Bound kinds minLength and min are incompatible" | "Bound kinds minLength and maxLength are incompatible" | "Bound kinds minLength and minLength are incompatible" | "Bound kinds minLength and before are incompatible" | "Bound kinds minLength and after are incompatible" | "Bound kinds before and exactLength are incompatible" | "Bound kinds before and max are incompatible" | "Bound kinds before and min are incompatible" | "Bound kinds before and maxLength are incompatible" | "Bound kinds before and minLength are incompatible" | "Bound kinds before and before are incompatible" | "Bound kinds before and after are incompatible" | "Bound kinds after and exactLength are incompatible" | "Bound kinds after and max are incompatible" | "Bound kinds after and min are incompatible" | "Bound kinds after and maxLength are incompatible" | "Bound kinds after and minLength are incompatible" | "Bound kinds after and before are incompatible" | "Bound kinds after and after are incompatible";
export declare const getBoundKinds: (comparator: Comparator, limit: LimitLiteral, root: RawSchema, boundKind: BoundExpressionKind) => BoundKind[];
export declare const singleEqualsMessage = "= is not a valid comparator. Use == to check for equality";
type singleEqualsMessage = typeof singleEqualsMessage;
export declare const parseRightBound: (s: DynamicStateWithRoot, comparator: Comparator) => void;
export type parseRightBound<s extends StaticState, comparator extends Comparator, $, args> = parseOperand<s, $, args> extends infer nextState extends StaticState ? nextState["root"] extends `${infer limit extends LimitLiteral}` ? s["branches"]["leftBound"] extends {} ? comparator extends MaxComparator ? state.reduceRange<s, s["branches"]["leftBound"]["limit"], s["branches"]["leftBound"]["comparator"], comparator, limit, nextState["unscanned"]> : state.error<writeUnpairableComparatorMessage<comparator>> : state.reduceSingleBound<s, comparator, limit, nextState["unscanned"]> : state.error<writeInvalidLimitMessage<comparator, astToString<nextState["root"]>, "right">> : never;
export declare const writeInvalidLimitMessage: <comparator extends "==" | "<" | "<=" | ">" | ">=", limit extends string | number, boundKind extends BoundExpressionKind>(comparator: comparator, limit: limit, boundKind: boundKind) => writeInvalidLimitMessage<comparator, limit, boundKind>;
export type writeInvalidLimitMessage<comparator extends Comparator, limit extends string | number, boundKind extends BoundExpressionKind> = `Comparator ${boundKind extends "left" ? InvertedComparators[comparator] : comparator} must be ${boundKind extends "left" ? "preceded" : "followed"} by a corresponding literal (was '${limit}')`;
export type BoundExpressionKind = "left" | "right";
export {};
